@@ -6,55 +6,31 @@ package AnyEvent::Handle::Throttle;
     use Errno qw[EAGAIN EINTR];
     use AnyEvent::Util qw[WSAEWOULDBLOCK];
     use parent 'AnyEvent::Handle';
-    our $MAJOR = 0.00; our $MINOR = 2; our $DEV = 3; our $VERSION = sprintf('%1.3f%03d' . ($DEV ? (($DEV < 0 ? '' : '_') . '%03d') : ('')), $MAJOR, $MINOR, abs $DEV);
+    our $MAJOR = 0.00; our $MINOR = 1; our $DEV = 0; our $VERSION = sprintf('%1.3f%03d' . ($DEV ? (($DEV < 0 ? '' : '_') . '%03d') : ('')), $MAJOR, $MINOR, abs $DEV);
 
-    sub upload_limit {
-        $_[1] ? $_[0]->{upload_limit} = $_[1] : $_[0]->{upload_limit};
+    sub upload_rate {
+        $_[1] ? $_[0]->{upload_rate} = $_[1] : $_[0]->{upload_rate};
     }
 
-    sub download_limit {
-        $_[1] ? $_[0]->{download_limit} = $_[1] : $_[0]->{download_limit};
+    sub download_rate {
+        $_[1] ? $_[0]->{download_rate} = $_[1] : $_[0]->{download_rate};
     }
-    sub upload_speed   { $_[0]->{upload_speed} }
-    sub download_speed { $_[0]->{download_speed} }
-    my ($global_upload_limit, $global_download_limit,
-        $global_upload_speed, $global_download_speed);
-    my $global_period = 1;
-
-    sub global_upload_limit {
-        $_[1] ? $global_upload_limit = $_[1] : $global_upload_limit;
-    }
-
-    sub global_download_limit {
-        $_[1] ? $global_download_limit = $_[1] : $global_download_limit;
-    }
-    sub global_upload_speed   {$global_upload_speed}
-    sub global_download_speed {$global_download_speed}
-    my ($global_read_size,     $global_write_size,
-        $global__upload_speed, $global__download_speed);
-    my $global_reset_cb = sub {
-        $global_read_size      = $global_download_limit;
-        $global_write_size     = $global_upload_limit || 8 * 1024;
-        $global_upload_speed   = $global__upload_speed;
-        $global_download_speed = $global__download_speed;
-        $global__upload_speed  = $global__download_speed = 0;
-    };
-    $global_reset_cb->();
-    our $global_reset = AE::timer(0, $global_period, $global_reset_cb);
+    sub upload_period   { $_[0]->{upload_period} }
+    sub download_period { $_[0]->{download_period} }
 
     sub _start {
-        my $self  = shift;
+        my $self = shift;
+        $self->SUPER::_start(@_);
         my $reset = sub {
-            $self->{read_size}      = $self->{download_limit};
-            $self->{write_size}     = $self->{upload_limit} || 8 * 1024;
-            $self->{upload_speed}   = $self->{_upload_speed};
-            $self->{download_speed} = $self->{_download_speed};
-            $self->{_upload_speed}  = $self->{_download_speed} = 0;
+            $self->{read_size}       = $self->{download_rate};
+            $self->{write_size}      = $self->{upload_rate} || 8 * 1024;
+            $self->{upload_period}   = $self->{_upload_period};
+            $self->{download_period} = $self->{_download_period};
+            $self->{_upload_period}  = $self->{_download_period} = 0;
         };
         $self->{_period} ||= 1;
         $self->{_reset} = AE::timer(0, $self->{_period}, $reset);
         $reset->();
-        $self->SUPER::_start(@_);
     }
 
     sub start_read {
@@ -62,14 +38,15 @@ package AnyEvent::Handle::Throttle;
         unless ($self->{_rw} || $self->{_eof} || !$self->{fh}) {
             Scalar::Util::weaken $self;
             $self->{_rw} = AE::io $self->{fh}, 0, sub {
-                my ($read) = sort grep {defined} $global_read_size,
-                    $self->{read_size};
-                my ($period) = sort grep {defined} $global_period,
-                    $self->{_period};
-                if (defined $read && $read <= 0) {
+                my $_read
+                    = defined $self->{download_rate}
+                    && defined $self->{read_size}
+                    ? $self->{read_size}
+                    : $self->{rbuf_max};
+                if (!$_read) {
                     $self->stop_read;
                     return $self->{_pause_read} = AE::timer(
-                        $period, 0,
+                        0.5, 0,
                         sub {
                             delete $self->{_pause_read};
                             $self->start_read;
@@ -78,13 +55,10 @@ package AnyEvent::Handle::Throttle;
                 }
                 my $rbuf = \($self->{tls} ? my $buf : $self->{rbuf});
                 $$rbuf ||= '';
-                my $len = sysread $self->{fh}, $$rbuf, $read || 8192,
-                    length $$rbuf;
-                if (defined $len && $len > 0) {
+                my $len = sysread $self->{fh}, $$rbuf, $_read, length $$rbuf;
+                if ($len > 0) {
                     $self->{read_size} -= $len;
-                    $global_read_size  -= $len;
-                    $self->{_download_speed} += $len;
-                    $global__download_speed  += $len;
+                    $self->{_download_period} += $len;
                     $self->{_activity} = $self->{_ractivity} = AE::now;
                     if ($self->{tls}) {
                         Net::SSLeay::BIO_write($self->{_rbio}, $$rbuf);
@@ -108,7 +82,7 @@ package AnyEvent::Handle::Throttle;
 
     sub _drain_wbuf {
         my ($self) = @_;
-        if (!$self->{_ww} && $self->{wbuf} && length $self->{wbuf}) {
+        if (!$self->{_ww} && length $self->{wbuf}) {
             Scalar::Util::weaken $self;
             my $cb;
             my $poll = sub {
@@ -116,11 +90,7 @@ package AnyEvent::Handle::Throttle;
                     if length $self->{wbuf};
             };
             $cb = sub {
-                my ($write) = sort grep {defined} $global_write_size,
-                    $self->{write_size};
-                my ($period) = sort grep {defined} $global_period,
-                    $self->{_period};
-                if (defined $write && $write <= 0) {
+                if (!$self->{write_size}) {
                     if (length $self->{wbuf}) {
                         delete $self->{_ww};
                         return $self->{_pause_ww} = AE::timer(
@@ -133,12 +103,11 @@ package AnyEvent::Handle::Throttle;
                     }
                     return 1;
                 }
-                my $len = syswrite $self->{fh}, $self->{wbuf}, $write;
+                my $len = syswrite $self->{fh}, $self->{wbuf},
+                    $self->{write_size};
                 if (defined $len) {
                     $self->{write_size} -= $len;
-                    $global_write_size  -= $len;
-                    $self->{_upload_speed} += $len;
-                    $global__upload_speed  += $len;
+                    $self->{_upload_period} += $len;
                     substr $self->{wbuf}, 0, $len, "";
                     $self->{_activity} = $self->{_wactivity} = AE::now;
                     $self->{on_drain}($self)
@@ -174,8 +143,8 @@ AnyEvent::Handle::Throttle - AnyEvent::Handle subclass with user-defined up/down
     my $condvar = AnyEvent->condvar;
     my $handle;
     $handle = AnyEvent::Handle::Throttle->new(
-        upload_limit   => 2,  # Very...
-        download_limit => 50, # ...slow
+        upload_rate   => 2,  # Very...
+        download_rate => 50, # ...slow
         connect  => ['google.com', 'http'],
         on_error => sub {
             warn "error $_[2]\n";
@@ -219,66 +188,38 @@ constructor supports these arguments (all as C<< key => value >> pairs).
 
 =over
 
-=item upload_limit => <bytes>
+=item upload_rate => <bytes>
 
-This is the maximum amount of data (in bytes) written to the filehandle per
-period. If C<upload_limit> is not specified, the upload rate is not limited.
+This is the maximum amount of data (in bytes) writen to the filehandle per
+period. If C<upload_rate> is not specified, the upload rate is not limited.
 
 Note that this value can/will override C<read_size>.
 
-=item download_limit => <bytes>
+=item download_rate => <bytes>
 
 This is the maximum amount of data (in bytes) read from the filehandle per
-period. If C<download_limit> is not specified, the upload rate is not limited.
+period. If C<download_rate> is not specified, the upload rate is not limited.
 
 =back
 
-=item $handle->B<upload_limit>( $bytes )
+=item $handle->B<upload_rate>( $bytes )
 
 Sets/returns the current upload rate in bytes per period.
 
-=item $handle->B<download_limit>( $bytes )
+=item $handle->B<download_rate>( $bytes )
 
 Sets/returns the current download rate in bytes per period.
 
-=item $bytes = $handle->B<upload_speed>( )
+=item $bytes = $handle->B<upload_period>( )
 
 Returns the amount of data written during the previous period.
 
-=item $bytes = $handle->B<download_speed>( )
+=item $bytes = $handle->B<download_period>( )
 
 Returns the amount of data read during the previous period.
 
 =back
 
-If you're using L<AnyEvent::Handle::Throttle|AnyEvent::Handle::Throttle> to
-limit bandwidth and realize you'd rather set flat limits on the B<total>
-bandwidth instead of per-handle, try these methods:
-
-=over
-
-=item AnyEvent::Handle::Throttle->B<global_upload_limit>( $bytes )
-
-Sets/returns the current global upload rate in bytes per period.
-
-=item AnyEvent::Handle::Throttle->B<global_download_limit>( $bytes )
-
-Sets/returns the current global download rate in bytes per period.
-
-=item $bytes = $handle->B<global_upload_speed>( )
-
-Returns the amount of data written through all
-L<AnyEvent::Handle::Throttle|AnyEvent::Handle::Throttle> objects during the
-previous period.
-
-=item $bytes = $handle->B<global_download_speed>( )
-
-Returns the amount of data read through all
-L<AnyEvent::Handle::Throttle|AnyEvent::Handle::Throttle> objects during the
-previous period.
-
-=back
-
 =head1 Notes
 
 =over
@@ -289,16 +230,16 @@ The current default period is C<1> second.
 
 =item *
 
-On destruction, all remaining data is sent ASAP, ignoring the user defined
+On destuction, all remaining data is sent ASAP, ignoring the user defined
 upload limit. This may change in the future.
 
 =back
 
 =head1 Bugs
 
-I'm sure this module is just burting with 'em. When you stumble upon one,
+I'm sure this module is just burting with 'em. When you stumble upon on,
 please report it via the
-L<Issue Tracker|http://github.com/sanko/anyevent-handle-throttle/issues>.
+L<Issue Tracker|http://github.com/sanko/anyevent-handle-bandwidth/issues>.
 
 =head1 Author
 
@@ -323,6 +264,6 @@ L<Creative Commons Attribution-Share Alike 3.0 License|http://creativecommons.or
 See the
 L<clarification of the CCA-SA3.0|http://creativecommons.org/licenses/by-sa/3.0/us/>.
 
-=for rcs $Id: Throttle.pm 8f0cb2d 2010-06-26 02:54:46Z sanko@cpan.org $
+=for rcs $Id: Throttle.pm a579491 2010-06-15 00:21:13Z sanko@cpan.org $
 
 =cut
